/*! DTAG FormReplace - v0.1.0 - 2013-03-27
 * Copyright (c) 2013 Jonas Ulrich | ruhmesmeile; Licensed MIT */
// use AMD if present
(function (factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define(['jquery'], factory);
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function ($) {

    $.fn.formReplace =
        function (options) {
            // Merge options into defaults
            $.fn.formReplace.opts = $.extend({}, $.fn.formReplace.defaults, options);

            // Iterate through form elements
            return this.each(function (index, formElement) {
                index = index; // do something with `index` due linting errors

                var $replacementElement = $($.fn.formReplace.opts.markup),
                    $formElement = $(formElement);

                // Move original form element out of viewport
                $(formElement).addClass('replaced-form-element')
                    .css({
                        'border': '0',
                        'clip': 'rect(0 0 0 0)',
                        'height': '1px',
                        'margin': '-1px',
                        'overflow': 'hidden',
                        'padding': '0',
                        'position': 'absolute',
                        'width': '1px'
                    });

                if ($formElement.is(':checked')) {
                    $formElement.parent().addClass('checked');
                }

                if ($formElement.is(':disabled')) {
                    $formElement.parent().addClass('disabled');
                }

                // Iterate over events & proxy them
                $.each(['click'], function (index, type) {
                    index = index; // do something with `index` due linting errors

                    $replacementElement.on(type, function (event) {
                        event.preventDefault();
                        if (!$.fn.formReplace.opts.doNotStopPropagation) {
                            event.stopPropagation();
                        }

                        if ($(this).parent().hasClass('disabled')) {
                            return;
                        }

                        var $targetElement = $formElement;
                        $targetElement.trigger($.Event(event));
                        $targetElement.blur();
                    });
                });

                $.each([!$.support.changeBubbles ? 'propertychange' : 'change'], function (index, type) {
                    index = index; // do something with `index` due linting errors

                    $formElement.on(type, function () {
                        var $fE = $(this);
                        if ($fE.is(':checked')) {
                            if ($fE.is('input[type=radio]')) {
                                $('input[name="' + $fE.prop('name') + '"]').parent().removeClass('checked');
                            }
                            $fE.parent().addClass('checked');
                        } else {
                            $fE.parent().removeClass('checked');
                        }
                    });
                });

                $formElement.on('focus', function () {
                    $replacementElement.addClass('hover');
                });

                $formElement.on('blur', function () {
                    $replacementElement.removeClass('hover');
                });

                // Insert replacement into DOM
                $formElement.after($replacementElement);
            });
        };

    /* Empty function for form elements that don't have any jQuery event-handlers */
    $.fn.formReplace.emptyFn = $.noop;

    /* Plugin defaults */
    $.fn.formReplace.defaults = {
        markup: '<span class="replacement-form-element"></span>'
    };

}));
