/**
 * @author Torsten Dinkheller
 *
 * Ping utility class
 *
 * initalize
 *
 *     Ping.setup(callback, [ip=config-env.WEBRTC.GW.IP], [timeSingle=500], [timeStart=1000])
 *
 * usage
 *
 *     Ping.single([callback])     starts a single ping, you may add a different callback
 *     Ping.start([callback])      starts looping, you may add a different callback
 *     Ping.stop()                 stops looping
 */
Ping = {
    callback: function () {},
    ip: null,
    img: null,
    startTime: new Date(),
    isRunning: false,
    timerSingle: 0,
    timeSingle: 0,
    timerLoop: 0,
    timeStart: 0,

    setup: function (callback, ip, timeSingle, timeStart) {
        Logger.log('[Ping::setup]');
        this.timeSingle = timeSingle || 500;
        this.timeStart = timeStart || 1000;
        this.ip = ip || require('config/Environment').getString('webRTC.host');
        this.callback = callback;

        if ((this.timerSingle + 200) > this.timeStart)
            Logger.warn('[Ping::setup] Please look over your timings: single-' + this.timeSingle + ' :: start-' + this.timeStart);
    },

    single: function (callback) {
        if (!this.ip && callback)
            this.setup(callback);

        if (!this.inUse) {
            Logger.log('[Ping::single] started');

            callback = callback || this.callback;
            this.startTime = new Date().getTime();

            var timeoutFn    = function () {
                callback(false);
            }.bind(this);
            this.timerSingle = setTimeout(timeoutFn, this.timeSingle);

            this.img = new Image();
            this.img.onload  = function () {
                callback(true);
                clearTimeout(this.timerSingle);
            }.bind(this);
            this.img.onerror = function (e) {
                callback(true, e);
                clearTimeout(this.timerSingle);
            }.bind(this);
            this.img.src = "http://" + this.ip;

            return 'pinging Server at ' + this.ip;
        } else {
            Logger.log('[Ping::single] NO new ping started');
        }
    },

    start: function (callback) {
        Logger.log('[Ping::start]');
        this.isRunning = true;
        this.loop(callback)
    },

    stop: function () {
        Logger.log('[Ping::stop]');
        this.isRunning = false;
        clearTimeout(this.timerLoop);
    },

    loop: function (callback) {
        callback = callback || this.callback;

        var timeoutFn = function () {
            this.loop(callback);
        }.bind(this);
        this.timerLoop = setTimeout(timeoutFn, this.timeStart);

        this.single(callback);
    }
};