cordova.define("de.isaco.webrtc.MediaStream", function(require, exports, module) {
var exec = require('cordova/exec'),
    utils = require('cordova/utils'),
    cordova = require('cordova'),
    log = require('./WebRTCLog');

var RTCTrackState = {
    0: "initializing",
    1: "live",
    2: "ended",
    3: "failed",
};

var MediaStream = function (streamId) {
  // insert object into cache
  this.id = streamId;
  MediaStream.__cache[this.id] = this;
  // event handlers defined in http://www.w3.org/TR/mediacapture-streams/
  this.onactive = null;
  this.oninactive = null;
  this.onaddtrack = null;
  this.onremovetrack = null;
  // attributes defined in http://www.w3.org/TR/mediacapture-streams/
  this.active = true;
  // private attributes
  this.__audioTracks = [];
  this.__videoTracks = [];
};

MediaStream.__cache = {};

MediaStream.__notify_event = function (event) {
  log.dump("__notify_event", JSON.stringify(event));
  var s = MediaStream.__cache[event.sid];
  if (!s)
    return;
  if (event.name === 'active') {
    
  } else if (event.name === 'inactive') {
    
  } else if (event.name === 'addtrack') {
    var t = event.track;
    var mst = new MediaStreamTrack(t.id, t.kind, RTCTrackState[t.state], Boolean(t.enabled), event.sid);
    if (t.kind === 'audio') {
      s.__audioTracks.push(mst);
    } else if (t.kind === 'video') {
      s.__videoTracks.push(mst);
    }
    var e = {
      track: mst,
      currentTarget: s
    };
    if (s.onaddtrack !== null) {
      s.onaddtrack(e);
    }
  } else if (event.name === 'removetrack') {
    
  } else if (event.name === 'destroy') {
    delete MediaStream.__cache[event.sid];
  }
};

MediaStream.getOrCreate = function (sid) {
  var s = MediaStream.__cache[sid];
  if (s !== undefined) {
    return s;
  }
  return new MediaStream(sid);
};

MediaStream.prototype.getAudioTracks = function () {
    return this.__audioTracks;
};

MediaStream.prototype.getVideoTracks = function () {
    return this.__videoTracks;
};

MediaStream.prototype.getTrackById = function (trackId) {
    var idx;
    for (idx = 0; idx < this.__audioTracks.length; idx++) {
        if (this.__audioTracks[idx].id === trackId) {
            return this.__audioTracks[idx];
        }
    }
    for (idx = 0; idx < this.__videoTracks.length; idx++) {
        if (this.__videoTracks[idx].id === trackId) {
            return this.__videoTracks[idx];
        }
    }
    return null;
};

MediaStream.prototype.addTrack = function (track) {
    log.debug("addTrack() not implemented yet");
};

MediaStream.prototype.removeTrack = function (track) {
    log.debug("removeTrack() not implemented yet");
};

MediaStream.prototype.clone = function () {
    log.debug("clone() not implemented yet");
};

module.exports = MediaStream;

});
