cordova.define("de.telekom.hometalk.callkit.CallKit", function(require, exports, module) {
var exec = require('cordova/exec'),
    Plugin = function (delegate) {

        var success = function(action) {

            var delegateAction = {
                callId: action.callId,

                fulfill: function() {
                    // TODO: do we need callbacks?
                    exec(undefined, undefined, 'CallKit', 'fulfillAction', [action.actionId]);
                },

                fail: function() {
                    // TODO: do we need callbacks?
                    exec(undefined, undefined, 'CallKit', 'failAction', [action.actionId]);
                }
            };

            var actionType = action.type;
            var methodName = 'on' + actionType.charAt(0).toUpperCase() + actionType.slice(1);

            var parameters = action.parameters;
            if (parameters) {
                for (var key in parameters) {
                    if (parameters.hasOwnProperty(key)) {
                        delegateAction[key] = parameters[key];
                    }
                }
            }

            delegate[methodName](delegateAction);
        };

        // TODO: do we need the fail callback?
        exec(success, undefined, 'CallKit', 'setCallbackHandler', []);
    };

/**
 *
 * @param number
 * @param displayName
 * @param ringtone
 * @param onSuccess returns the id of the call that is required by other functions of this plugin
 * @param onError
 */
Plugin.prototype.handleIncomingCall = function (number, displayName, ringtone, onSuccess, onError) {
    return exec(onSuccess, onError, 'CallKit', 'handleIncomingCall', [number, displayName, ringtone]);
};

/**
 *
 * @param number
 * @param displayName
 * @param onSuccess returns the id of the call that is required by other functions of this plugin
 * @param onError
 */
Plugin.prototype.handleOutgoingCall = function (number, displayName, onSuccess, onError) {
    return exec(onSuccess, onError, 'CallKit', 'handleOutgoingCall', [number, displayName]);
};

/**
 *
 * @param callId the id returned by {@link #requestOutgoingCall}
 * @param onSuccess
 * @param onError
 */
Plugin.prototype.outgoingCallConnecting = function (callId, onSuccess, onError) {
    return exec(onSuccess, onError, 'CallKit', 'outgoingCallConnecting', [callId]);
};

/**
 *
 * @param callId the id returned by {@link #requestOutgoingCall}
 * @param onSuccess
 * @param onError
 */
Plugin.prototype.outgoingCallConnected = function (callId, onSuccess, onError) {
    return exec(onSuccess, onError, 'CallKit', 'outgoingCallConnected', [callId]);
};

/**
 *
 * @param callId the id returned by {@link #showIncomingCall}
 * @param onSuccess
 * @param onError
 */
Plugin.prototype.answerCall = function (callId, onSuccess, onError) {
    return exec(onSuccess, onError, 'CallKit', 'answerCall', [callId]);
};

/**
 *
 * @param callId the id returned by {@link #requestOutgoingCall} or {@link #showIncomingCall}
 * @param onSuccess
 * @param onError
 */
Plugin.prototype.endCall = function (callId, onSuccess, onError) {
    return exec(onSuccess, onError, 'CallKit', 'endCall', [callId]);
};

/**
 *
 * @param callId the id returned by {@link #requestOutgoingCall} or {@link #showIncomingCall}
 * @param reason
 * @param onSuccess
 * @param onError
 */
Plugin.prototype.callEnded = function (callId, reason, onSuccess, onError) {
    return exec(onSuccess, onError, 'CallKit', 'callEnded', [callId, reason]);
};

/**
 *
 * @param callId the id returned by {@link #requestOutgoingCall} or {@link #showIncomingCall}
 * @param muted
 * @param onSuccess
 * @param onError
 */
Plugin.prototype.muted = function (callId, muted, onSuccess, onError) {
    return exec(onSuccess, onError, 'CallKit', 'muted', [callId, muted]);
};

/**
 *
 * @param callId the id returned by {@link #requestOutgoingCall} or {@link #showIncomingCall}
 * @param digit
 * @param onSuccess
 * @param onError
 */
Plugin.prototype.dtmf = function (callId, digit, onSuccess, onError) {
    return exec(onSuccess, onError, 'CallKit', 'dtmf', [callId, digit]);
};

module.exports = {

    /**
     * checks whether or not the CallKit framework is supported on this device or not
     * @param onSuccess
     * @param onError
     */
    isSupported: function (onSuccess, onError) {
        return exec(onSuccess, onError, 'CallKit', 'isSupported', []);
    },

    registerOutgoingCallHandler: function (callback) {
        return exec(callback, undefined, 'CallKit', 'registerOutgoingCallHandler', []);
    },

    /**
     *
     * @param delegate object with various callback functions.
     * @returns {Plugin}
     */
    init: function(delegate) {
        return new Plugin(delegate);
    },

    endReason: {
        failed: 0,
        endedOnRemoteSide: 1,
        unanswered: 2,
        answeredElsewhere: 3,
        declinedElsewhere: 4
    }

};
});
