define([
    'underscore'
],
    function(_) {

        return {

            updatesToVersion: function() {
                return '2.0.1';
            },

            /**
             *
             * @param moduleContainer to grant access to modules. Modules should try to only use the local storage and keychain modules.
             * @param callback will be invoked after executing this step. First parameter (boolean) indicates whether or not there was a problem during the execution.
             */
            execute: function(moduleContainer, callback) {
                var localStorageProperties = [
                    {name: 'app-model-data', persistent: false},
                    {name: 'cached-faq', persistent: false},
                    {name: 'connection-data', persistent: false},
                    {name: 'favorites-data', persistent: false},
                    {name: 'first-time-usage', persistent: true},
                    {name: 'history', persistent: false},
                    {name: 'last-route', persistent: false},
                    {name: 'offline-phonenumbers', persistent: false},
                    {name: 'recall.number', persistent: false},
                    {name: 'sam3AccessToken', persistent: false},
                    {name: 'sam3RefreshToken', persistent: false},
                    {name: 'sip-instance-id', persistent: false},
                    {name: 'tokenExpireTimeout', persistent: false}
                ];
                _.each(localStorageProperties, _.partial(this.reformatStorageProperty, moduleContainer, _));

                var keyChainProperties = [
                    'pass',
                    'username'
                ];
                _.each(keyChainProperties, _.partial(this.fromLocalStorageToKeyChainProperty, moduleContainer, _));

                var unmanagedProperties = [
                    {name: 'puno.deviceId', persistent: true},
                    {name: 'puno.osId', persistent: true},
                    {name: 'puno.punoToken', persistent: false},
                    {name: 'puno.sipId', persistent: false}
                ];
                _.each(unmanagedProperties, _.partial(this.toManagedProperty, moduleContainer, _));

                callback(true);
            },

            reformatStorageProperty: function(moduleContainer, property) {
                Logger.log('[Migration_to_2.0.1::reformatStorageProperties] converting property ' + property.name);
                // read raw value. we can't use the local storage module, because the current value is stored in an invalid format
                var value = localStorage.getItem('__amplify__hometalk.' + property.name);

                if (value) {
                    value = JSON.parse(value).data;
                    Logger.log('[Migration_to_2.0.1::reformatStorageProperties] value is ' + value);
                    // decrypt value
                    value = moduleContainer.Crypto.decrypt(value);
                    Logger.log('[Migration_to_2.0.1::reformatStorageProperties] decrypted value is ' + value);
                    // and add it to the storage again
                    moduleContainer.LocalStorageModule.store(property.name, value, {persistent: property.persistent});
                }
            },

            fromLocalStorageToKeyChainProperty: function(moduleContainer, name) {
                Logger.log('[Migration_to_2.0.1::fromLocalStorageToKeyChain] converting property ' + name);
                // read raw value. we can't use the local storage module, because the current value is stored in an invalid format
                var value = localStorage.getItem('__amplify__hometalk.' + name);

                if (value) {
                    value = JSON.parse(value).data;
                    Logger.log('[Migration_to_2.0.1::fromLocalStorageToKeyChain] value is ' + value);
                    // decrypt value
                    value = moduleContainer.Crypto.decrypt(value);
                    Logger.log('[Migration_to_2.0.1::fromLocalStorageToKeyChain] decrypted value is ' + value);
                    // and add it to the key chain
                    moduleContainer.KeyChainModule.store(name, value);
                }
            },

            toManagedProperty: function(moduleContainer, property) {
                Logger.log('[Migration_to_2.0.1::toManagedProperty] converting property ' + property.name);
                var value = localStorage.getItem(property.name);

                if (value) {
                    Logger.log('[Migration_to_2.0.1::toManagedProperty] value is ' + value);
                    // remove the old property
                    localStorage.removeItem(property.name);
                    // and add it to the storage again
                    moduleContainer.LocalStorageModule.store(property.name, value, {persistent: property.persistent});
                }

            }
        };

});