define([
        'underscore',
        // Use this provider instead of asking the current login module for its token provider.
        'features/credentialLogin/CredentialTokenProvider'
    ],
    function(_, TokenProvider) {

        return {

            updatesToVersion: function() {
                return '2.1.0'; // NEVER change this version
            },

            /**
             *
             * @param moduleContainer to grant access to modules. Modules should try to only use the local storage and keychain modules.
             * @param callback will be invoked after executing this step. First parameter (boolean) indicates whether or not there was a problem during the execution.
             */
            execute: function(moduleContainer, callback) {
                var firstTimeUsageProperty = moduleContainer.LocalStorageModule.get('first-time-usage');
                if (!firstTimeUsageProperty) {
                    // if this property doesn't exist, the user is not logged in
                    callback(true);
                    return;
                }

                this.migrateLoginProperties(moduleContainer, callback, firstTimeUsageProperty);
            },

            migrateLoginProperties: function(moduleContainer, callback, firstTimeUsageProperty) {
                var lastPage = firstTimeUsageProperty.lastPage;
                if (lastPage && lastPage !== 'undefined' && lastPage !== 'allPermission-login-container' && lastPage !== 'finished-login-container') {
                    // if lastPage is set to an unknown value (the string 'undefined' is an expected value), don't migrate anything
                    callback(true);
                    return;
                }

                // we should have valid user credentials at this point.
                var userName;
                var password;
                var credentialsAvailable = _.after(2, _.bind(function() {
                    this.fetchTokens(userName, password, moduleContainer, callback, firstTimeUsageProperty);
                }, this));
                var abort = function() { callback(false); };

                moduleContainer.KeyChainModule.get('username', function(value) {
                    userName = value;
                    credentialsAvailable();
                }, abort);
                moduleContainer.KeyChainModule.get('pass', function(value) {
                    password = value;
                    credentialsAvailable();
                }, abort);
            },

            fetchTokens: function(userName, password, moduleContainer, callback, firstTimeUsageProperty) {
                if (!userName || !password) {
                    callback(false);
                    return;
                }

                var storage = moduleContainer.LocalStorageModule;

                // it's possible that the provider is not yet initialized (e.g. if later versions of the app uses different login modules)
                TokenProvider.setStorage(storage);

                TokenProvider.retrieveTokens(userName, password).then(
                    function() {
                        var lastPage; // undefined is the default value if the login process is finished
                        var lastRoute = 'dialpad';

                        switch (firstTimeUsageProperty.lastPage) {
                            case 'allPermission-login-container':
                                lastPage = '#loginPermission';
                                lastRoute = 'loginPermission';
                                break;
                            case 'finished-login-container':
                                lastPage = '#loginFinished';
                                lastRoute = 'loginFinished';
                                break;
                        }

                        storage.store('login-process-status', {lastPage: lastPage, firstTime: firstTimeUsageProperty.firstTime}, {persistent: true});
                        storage.store('last-route', lastRoute);

                        storage.remove('first-time-usage');
                        storage.remove('sam3AccessToken');
                        storage.remove('sam3RefreshToken');
                        storage.remove('tokenExpireTimeout');
                        callback(true);
                    },
                    function() {
                        callback(false);
                    }
                );
            }
        };

    });